/////////////////////////////////////////////////////////////////
//   Title: IR-Remote Control 
// 
//   Author: Unknown & Various Sources 
//   Co-Author: Abra-Electronics M.P Division
//
//   Description: This software is designed with the intent to
//   communicate commands using the IR Sensor. Should you get a 
//   compilation error as: int RECV_PIN = TDK2, please go to  
//   my programs located within the Operating system folder. 
//   Open Arduino> libraries> RobotIRremote> src remove the files
//   IRremoteTools.h & IRremoteTools.cpp. Close this windows and 
//   any window containing arduino and reopen this file. You should
//   be able to compile it!
//
//   Date: 9/1/2015 @ 1:54 PM EDT
//
/////////////////////////////////////////////////////////////////

//--------------------------------------  Configuration Pinouts
#include <IRremote.h>
int RECV_PIN = 3;                  // Pinout IR sensor

int motor_left[] = {9,6};          // Pinout Left motors (Front, Back)
int motor_right[] = {11,10};       // Pinout Right motors (Front, Back)

long forward = 0x00FF18E7;         // number 2 on Remote Control
long backward = 0x00FF4AB5;         // number 8 on Remote Control
long brake = 0x00FF38C7;           // number 5 on Remote Control
long left = 0x00FF10EF;            // number 4 on Remote Control
long right = 0x00FF5AA5;           // number 6 on Remote Control

//--------------------------------------  Configuration IR 
IRrecv irrecv(RECV_PIN);
decode_results results;

void dump(decode_results *results) 
{
    int count = results->rawlen;
    if (results->decode_type == UNKNOWN) 
    {
        Serial.println("Could not decode message");
    } 
    else 
    {
        if (results->decode_type == NEC) 
        {
            Serial.print("Decoded NEC: ");
        } 
        else if (results->decode_type == SONY) 
        {
           Serial.print("Decoded SONY: ");
        } 
        else if (results->decode_type == RC5) 
        {
           Serial.print("Decoded RC5: ");
        } 
        else if (results->decode_type == RC6) 
        {
           Serial.print("Decoded RC6: ");
        }
         Serial.print(results->value, HEX);
         Serial.print(" (");
         Serial.print(results->bits, DEC);
         Serial.println(" bits)");
     }
     Serial.print("Raw (");
     Serial.print(count, DEC);
     Serial.print("): ");

     for (int i = 0; i < count; i++) 
     {
         if ((i % 2) == 1) 
         {
             Serial.print(results->rawbuf[i]*USECPERTICK, DEC);
         } 
         else  
         {
             Serial.print(-(int)results->rawbuf[i]*USECPERTICK, DEC);
         }
         Serial.print(" ");
     }
     Serial.println("");
}

//--------------------------------------  Setup Module
void setup()
{
    Serial.begin(9600);
    
    pinMode(RECV_PIN, INPUT);  
    
    for(int i=0; i<2; i++)
    {
       pinMode(motor_left[i],OUTPUT);  // set pins as output left
       pinMode(motor_right[i],OUTPUT); // set pins as output right
    }
 
    irrecv.enableIRIn();               // Start the receiver
}

//--------------------------------------  Variable declaration 
int on = 0;
unsigned long last = millis();

//--------------------------------------  Loop Module
void loop() 
{
    if (irrecv.decode(&results)) 
    {
    // If it's been at least 1/4 second since the last
    // IR received, toggle the relay
        if (millis() - last > 250) 
        {
            on = !on;

           digitalWrite(13, on ? HIGH : LOW);
           dump(&results);
        }
        
        if (results.value == forward)
        {
            straight();
        }

        if (results.value == backward )
        {
            reverse();
        }

        if (results.value == left )
        { 
            lefty();
        }


        if (results.value == right )
        { 
             righty();
        }

        if (results.value == brake )
        {
             halt();
        } 
          
    last = millis();      
    irrecv.resume(); // Receive the next value
  }
}

//-------------------------------------- Function Configuration
void straight()
{
    analogWrite(motor_right[0],0);
    analogWrite(motor_right[1],80);
    
    analogWrite(motor_left[0],0);
    analogWrite(motor_left[1],80);
}

void reverse()
{
    analogWrite(motor_right[0],80);
    analogWrite(motor_right[1],0); 
    
    analogWrite(motor_left[0],80);
    analogWrite(motor_left[1],0);
}

void lefty()
{
    analogWrite(motor_right[0],0);
    analogWrite(motor_right[1],0); 
    
    analogWrite(motor_left[0],0);
    analogWrite(motor_left[1],80);
}

void righty()
{
    analogWrite(motor_right[0],0);
    analogWrite(motor_right[1],80);
    
    analogWrite(motor_left[0],0);
    analogWrite(motor_left[1],0);
}

void halt()
{
   digitalWrite(motor_left[0],LOW);  
   digitalWrite(motor_left[1],LOW);  
  
   digitalWrite(motor_right[0],LOW);
   digitalWrite(motor_right[1],LOW);
   
   delay(25);
}
